@@ -143,13 +143,15 @@ class Proxy():
 
     async def _handle_command_reply(self, fu):
 
-        ret = await fu
+        reply = await fu
 
-        if hasattr(ret, "status") and ret.status.did_fail:
-            raise self._errorMapToException(ret.replies[-1].message['error'])
+        if hasattr(reply, "status") and reply.status.did_fail:
+            raise self._errorMapToException(reply.replies[-1].message['error'])
 
-        return ProxyDict(ret.replies[-1].message)
+        msg = ProxyDict(reply.replies[-1].message)
+        msg.sender = reply.actor
 
+        return msg
 
     def _handle_callback(self, callback: Optional[Callable[[AMQPReply], None]], reply: AMQPReply):
         msg = ProxyDict(json.loads(reply.message.body))
@@ -163,6 +165,7 @@ class Proxy():
                            callback: Optional[Callable[[dict], None]] = None,
                            time_limit: Optional[float] = 42.0,
                            nowait:Bool = False,
+                           nosync:Bool = False,
                            object_hook: Optional[Callable[[AMQPReply], None]] = None,
                            **kwargs):
 
@@ -176,13 +179,14 @@ class Proxy():
                                         for k, v in kwargs.items()))
 
         fu = await self.amqpc.send_command(self.actor,
-                                                command,
-                                                *args,
-                                                callback=partial(self._handle_callback, callback) if callback else None,
-                                                time_limit=time_limit)
+                                           command,
+                                           *args,
+                                           callback=partial(self._handle_callback, callback) if callback else None,
+                                           time_limit=time_limit)
         
 
 
+        if nosync: return
         if nowait: return self._handle_command_reply(fu)
 
         return await self._handle_command_reply(fu)
@@ -222,14 +226,23 @@ async def invoke(*cmds, return_exceptions:Bool=False):
     with an exception and return values for every command.
     """
     
+    actors=[c.cr_frame.f_locals['self'].actor for c in cmds]
+
     ret = await asyncio.gather(*cmds, return_exceptions=True)
 
-    ret = ProxyListOfDicts([ProxyDict(r) if isinstance(r, dict) else r for r in ret])
+    def _format(r):
+        if isinstance(r, dict): return ProxyDict(r)
+        elif isinstance(r, Exception): return ProxyDict({'error': r}) 
+        else: return r
+
+    ret = ProxyListOfDicts([_format(r) for r in ret])
+    ret.actors = actors
 
     if not return_exceptions:
         for r in ret:
-            if isinstance(r, Exception):
+            if isinstance(r, dict) and "error" in r.keys():
                 raise ProxyPartialInvokeException(*ret)
+
     return ret
 
 
@@ -304,6 +317,7 @@ def flatten(d: MutableMapping, parent_key: str = '', sep: str = '.'):
 
 class ProxyDict(dict):
    """ Extra helper class for the reply dict """
+
    def flatten(self):
         return flatten(self)
 
@@ -319,3 +333,6 @@ class ProxyListOfDicts(list):
    def unpack(self, *keys):
         return unpack(self, *keys)
 
+   def with_actors(self):
+        return dict(zip(self.actors, self))
+
